/** @file talker.h */
/* 
 * Copyright (C) 2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

#ifndef _TALKER_H
#define _TALKER_H

#include "graphbuilder.h"
#include "sampler.h"
#include "pvminterface.h"
#include <algorithm>
#include <sstream>

/// An array of these is sent to synchronize simulations
struct LeafCountsStruct {
  uint32 id;
  uint32 occupation_count;
};

/// Used for sorting
/* struct less_LeafCountsStruct: public binary_function<LeafCountsStruct,LeafCountsStruct,bool> { */
/*   bool operator()(LeafCountsStruct x, LeafCountsStruct y) {  */
/*     return (x.id < y.id); */
/*   } */
/* }; */

#define MAXLEAFMESSAGE_SIZE ((5 * Mb)/sizeof(LeafCountsStruct))

/// Reads and executes interactive user commands.
class Talker {
 public:
  Talker(const char* rippername, const char* tempdir,
	 WebLinkGraph *ripperweb, GraphBuilder *rippergb,
	 bool pvm_is_master, int pvm_numtasks) throw (exception);
  ~Talker();

  void Talk() throw (exception);

  void PropagateLeafCounts() throw (invalid_argument);

  void PrintStatistics(ostream& o);
  void PrintStatisticsGraph(ostream& o);

  bool BuildTags(ifstream& g);
  bool LoadLeaves(ifstream& g);

  int GetToken(char* command, ostream& out, char* message, int n, ...);
  char *GetTokenVar(char* command, ostream& out, char* message);
  bool ProcessCommand(char *command, ostream& out);

  void SetupFIFOs() throw (exception);
  void RemoveFIFOs();

 private:

  ostringstream inputfifo;
  ostringstream outputfifo;
  ostringstream nodequiv;
  const char *name;
  const char *tempdir;
  WebLinkGraph *web;
  GraphBuilder *gb;

  gsl_rng* r; // for random numbers

  PageRankSampler *prsampler;
  DateBiasedPageRankSampler *dbsampler;
  TruncatedKleinbergSampler *tksampler;
  WebSampler *defaultsampler;

  PVMInterface *pvminterface;

  LeafNodePtrList *leaflist;
};

#endif
